@@ -39,14 +39,15 @@ SSG_REF_URIS = {
 }
 
 product_directories = [
+    'alinux2',
+    'alinux3',
+    'anolis8',
     'chromium',
-    'debian9', 'debian10', 'debian11',
+    'debian10', 'debian11',
     'example',
     'eks',
     'fedora',
     'firefox',
-    'fuse6',
-    'jre',
     'macos1015',
     'ocp4',
     'rhcos4',
@@ -55,9 +56,8 @@ product_directories = [
     'rhel7', 'rhel8', 'rhel9',
     'rhv4',
     'sle12', 'sle15',
-    'ubuntu1604', 'ubuntu1804', 'ubuntu2004',
-    'vsel',
-    'wrlinux8', 'wrlinux1019'
+    'ubuntu1604', 'ubuntu1804', 'ubuntu2004', 'ubuntu2204',
+    'uos20',
 ]
 
 JINJA_MACROS_DIRECTORY = os.path.join(os.path.dirname(os.path.dirname(
@@ -98,9 +98,11 @@ ssg_version_uri = \
     "https://github.com/ComplianceAsCode/content/releases/latest"
 OSCAP_VENDOR = "org.ssgproject"
 OSCAP_DS_STRING = "xccdf_%s.content_benchmark_" % OSCAP_VENDOR
+OSCAP_BENCHMARK = "xccdf_%s.content_benchmark_" % OSCAP_VENDOR
 OSCAP_PROFILE = "xccdf_%s.content_profile_" % OSCAP_VENDOR
 OSCAP_GROUP = "xccdf_%s.content_group_" % OSCAP_VENDOR
 OSCAP_RULE = "xccdf_%s.content_rule_" % OSCAP_VENDOR
+OSCAP_VALUE = "xccdf_%s.content_value_" % OSCAP_VENDOR
 OSCAP_GROUP_PCIDSS = "xccdf_%s.content_group_pcidss-req" % OSCAP_VENDOR
 OSCAP_GROUP_VAL = "xccdf_%s.content_group_values" % OSCAP_VENDOR
 OSCAP_GROUP_NON_PCI = "xccdf_%s.content_group_non-pci-dss" % OSCAP_VENDOR
@@ -191,19 +193,18 @@ PKG_MANAGER_TO_CONFIG_FILE = {
 }
 
 FULL_NAME_TO_PRODUCT_MAPPING = {
+    "Alibaba Cloud Linux 2": "alinux2",
+    "Alibaba Cloud Linux 3": "alinux3",
+    "Anolis OS 8": "anolis8",
     "Chromium": "chromium",
-    "Debian 9": "debian9",
     "Debian 10": "debian10",
     "Debian 11": "debian11",
     "Example": "example",
     "Amazon Elastic Kubernetes Service": "eks",
     "Fedora": "fedora",
     "Firefox": "firefox",
-    "JBoss Fuse 6": "fuse6",
-    "Java Runtime Environment": "jre",
     "Apple macOS 10.15": "macos1015",
     "Red Hat OpenShift Container Platform 4": "ocp4",
-    "McAfee VirusScan Enterprise for Linux": "vsel",
     "Red Hat Enterprise Linux CoreOS 4": "rhcos4",
     "Oracle Linux 7": "ol7",
     "Oracle Linux 8": "ol8",
@@ -218,8 +219,9 @@ FULL_NAME_TO_PRODUCT_MAPPING = {
     "Ubuntu 16.04": "ubuntu1604",
     "Ubuntu 18.04": "ubuntu1804",
     "Ubuntu 20.04": "ubuntu2004",
-    "WRLinux 8": "wrlinux8",
-    "WRLinux 1019": "wrlinux1019",
+    "Ubuntu 22.04": "ubuntu2204",
+    "UnionTech OS Server 20": "uos20",
+    "Not Applicable" : "example"
 }
 
 
@@ -254,15 +256,21 @@ REFERENCES = dict(
     pcidss=Reference(
         id="pcidss", name=REF_PREFIX_MAP["pcidss"], url="",
         regex_with_groups=r"Req-(\d+)(?:\.(\w+)(?:\.(\w+)(?:\.(\w+))?)?)?"),
+    srg=Reference(
+        id="srg", name="SRG", url="",
+        regex_with_groups=r"(SRG-OS-\d+-GPOS-\d+)"
+    )
 )
 
 
 MULTI_PLATFORM_LIST = ["rhel", "fedora", "rhv", "debian", "ubuntu",
-                       "wrlinux", "opensuse", "sle", "ol", "ocp", "rhcos",
-                       "example", "eks"]
+                       "opensuse", "sle", "ol", "ocp", "rhcos",
+                       "example", "eks", "alinux", "uos", "anolis"]
 
 MULTI_PLATFORM_MAPPING = {
-    "multi_platform_debian": ["debian9", "debian10", "debian11"],
+    "multi_platform_alinux": ["alinux2", "alinux3"],
+    "multi_platform_anolis": ["anolis8"],
+    "multi_platform_debian": ["debian10", "debian11"],
     "multi_platform_example": ["example"],
     "multi_platform_eks": ["eks"],
     "multi_platform_fedora": ["fedora"],
@@ -273,19 +281,17 @@ MULTI_PLATFORM_MAPPING = {
     "multi_platform_rhel": ["rhel7", "rhel8", "rhel9"],
     "multi_platform_rhv": ["rhv4"],
     "multi_platform_sle": ["sle12", "sle15"],
-    "multi_platform_ubuntu": ["ubuntu1604", "ubuntu1804", "ubuntu2004"],
-    "multi_platform_wrlinux": ["wrlinux8", "wrlinux1019"],
+    "multi_platform_ubuntu": ["ubuntu1604", "ubuntu1804", "ubuntu2004", "ubuntu2204"],
+    "multi_platform_uos": ["uos20"],
 }
 
 RHEL_CENTOS_CPE_MAPPING = {
-    "cpe:/o:redhat:enterprise_linux:6": "cpe:/o:centos:centos:6",
     "cpe:/o:redhat:enterprise_linux:7": "cpe:/o:centos:centos:7",
     "cpe:/o:redhat:enterprise_linux:8": "cpe:/o:centos:centos:8",
     "cpe:/o:redhat:enterprise_linux:9": "cpe:/o:centos:centos:9",
 }
 
 RHEL_SL_CPE_MAPPING = {
-    "cpe:/o:redhat:enterprise_linux:6": "cpe:/o:scientificlinux:scientificlinux:6",
     "cpe:/o:redhat:enterprise_linux:7": "cpe:/o:scientificlinux:scientificlinux:7",
 }
 
@@ -426,21 +432,21 @@ XCCDF_PLATFORM_TO_PACKAGE = {
 
 # _version_name_map = {
 MAKEFILE_ID_TO_PRODUCT_MAP = {
+    'alinux': 'Alibaba Cloud Linux',
+    'anolis': 'Anolis OS',
     'chromium': 'Google Chromium Browser',
     'fedora': 'Fedora',
     'firefox': 'Mozilla Firefox',
-    'jre': 'Java Runtime Environment',
     'macos': 'Apple macOS',
     'rhel': 'Red Hat Enterprise Linux',
     'rhv': 'Red Hat Virtualization',
     'debian': 'Debian',
     'ubuntu': 'Ubuntu',
+    'uos': 'UnionTech OS Server',
     'eap': 'JBoss Enterprise Application Platform',
     'fuse': 'JBoss Fuse',
     'opensuse': 'openSUSE',
     'sle': 'SUSE Linux Enterprise',
-    'vsel': 'McAfee VirusScan Enterprise for Linux',
-    'wrlinux': 'WRLinux',
     'example': 'Example',
     'ol': 'Oracle Linux',
     'ocp': 'Red Hat OpenShift Container Platform',
@@ -462,3 +468,4 @@ DEFAULT_AIDE_BIN_PATH = '/usr/sbin/aide'
 DEFAULT_SSH_DISTRIBUTED_CONFIG = 'false'
 DEFAULT_PRODUCT = 'example'
 DEFAULT_CHRONY_CONF_PATH = '/etc/chrony.conf'
+DEFAULT_AUDISP_CONF_PATH = '/etc/audit'
