@@ -21,14 +21,18 @@ from ssg_test_suite import test_env
 from ssg_test_suite import common
 from ssg_test_suite.log import LogHelper
 
+import ssg.templates
+
+Rule = collections.namedtuple(
+    "Rule",
+    ["directory", "id", "short_id", "template", "local_env_yaml", "rule"])
+
+RuleTestContent = collections.namedtuple(
+    "RuleTestContent", ["scenarios", "other_content"])
 
 logging.getLogger(__name__).addHandler(logging.NullHandler())
 
 
-Scenario = collections.namedtuple(
-    "Scenario", ["script", "context", "script_params", "contents"])
-
-
 def get_viable_profiles(selected_profiles, datastream, benchmark, script=None):
     """Read datastream, and return set intersection of profiles of given
     benchmark and those provided in `selected_profiles` parameter.
@@ -86,8 +90,8 @@ def _apply_script(rule_dir, test_env, script):
 
         try:
             error_msg_template = (
-                f"Rule '{rule_name}' test setup script '{script}' "
-                "failed with exit code {rc}"
+                "Rule '{rule_name}' test setup script '{script}' "
+                "failed with exit code {{rc}}".format(rule_name=rule_name, script=script)
             )
             test_env.execute_ssh_command(
                 command, log_file, error_msg_template=error_msg_template)
@@ -97,14 +101,6 @@ def _apply_script(rule_dir, test_env, script):
     return True
 
 
-def _get_script_context(script):
-    """Return context of the script."""
-    result = re.search(r'.*\.([^.]*)\.[^.]*$', script)
-    if result is None:
-        return None
-    return result.group(1)
-
-
 class RuleChecker(oscap.Checker):
     """
     Rule checks generally work like this -
@@ -129,6 +125,10 @@ class RuleChecker(oscap.Checker):
         self._current_result = None
         self.remote_dir = ""
         self.target_type = "rule ID"
+        self.used_templated_test_scenarios = collections.defaultdict(set)
+        self.rule_spec = None
+        self.template_spec = None
+        self.scenarios_profile = None
 
     def _run_test(self, profile, test_data):
         scenario = test_data["scenario"]
@@ -219,68 +219,116 @@ class RuleChecker(oscap.Checker):
             logging.error(msg)
         return success
 
-    def _rule_template_been_tested(self, rule, tested_templates):
-        if rule.template is None:
-            return False
-        if self.test_env.duplicate_templates:
-            return False
-        if rule.template in tested_templates:
-            return True
-        tested_templates.add(rule.template)
-        return False
 
-    def _rule_should_be_tested(self, rule, rules_to_be_tested, tested_templates):
-        if 'ALL' in rules_to_be_tested:
-            # don't select rules that are not present in benchmark
-            if not xml_operations.find_rule_in_benchmark(
-                    self.datastream, self.benchmark_id, rule.id):
-                return False
-            return not self._rule_template_been_tested(rule, tested_templates)
+    def _rule_matches_rule_spec(self, rule_short_id):
+        rule_id = OSCAP_RULE + rule_short_id
+        if 'ALL' in self.rule_spec:
+            return True
         else:
-            for rule_to_be_tested in rules_to_be_tested:
+            for rule_to_be_tested in self.rule_spec:
                 # we check for a substring
                 if rule_to_be_tested.startswith(OSCAP_RULE):
                     pattern = rule_to_be_tested
                 else:
                     pattern = OSCAP_RULE + rule_to_be_tested
-                if fnmatch.fnmatch(rule.id, pattern):
-                    return not self._rule_template_been_tested(rule, tested_templates)
+                if fnmatch.fnmatch(rule_id, pattern):
+                    return True
             return False
 
-    def _ensure_package_present_for_all_scenarios(self, scenarios_by_rule):
+    def _rule_matches_template_spec(self, template):
+        return True
+
+    def _ensure_package_present_for_all_scenarios(
+            self, test_content_by_rule_id):
         packages_required = set()
-        for rule, scenarios in scenarios_by_rule.items():
-            for s in scenarios:
+        for rule_test_content in test_content_by_rule_id.values():
+            for s in rule_test_content.scenarios:
                 scenario_packages = s.script_params["packages"]
                 packages_required.update(scenario_packages)
         if packages_required:
             common.install_packages(self.test_env, packages_required)
 
-    def _prepare_environment(self, scenarios_by_rule):
-        domain_ip = self.test_env.domain_ip
+    def _prepare_environment(self, test_content_by_rule_id):
         try:
-            self.remote_dir = common.send_scripts(self.test_env)
+            self.remote_dir = common.send_scripts(
+                self.test_env, test_content_by_rule_id)
         except RuntimeError as exc:
             msg = "Unable to upload test scripts: {more_info}".format(more_info=str(exc))
             raise RuntimeError(msg)
 
-        self._ensure_package_present_for_all_scenarios(scenarios_by_rule)
+        self._ensure_package_present_for_all_scenarios(test_content_by_rule_id)
 
-    def _get_rules_to_test(self, target):
-        rules_to_test = []
-        tested_templates = set()
-        for rule in common.iterate_over_rules(self.test_env.product):
-            if not self._rule_should_be_tested(rule, target, tested_templates):
-                continue
-            if not xml_operations.find_rule_in_benchmark(
-                    self.datastream, self.benchmark_id, rule.id):
-                logging.error(
-                    "Rule '{0}' isn't present in benchmark '{1}' in '{2}'"
-                    .format(rule.id, self.benchmark_id, self.datastream))
-                continue
-            rules_to_test.append(rule)
+    def _get_rules_to_test(self):
+        """
+        Returns:
+            List of named tuples Rule having these fields:
+                directory -- absolute path to the rule "tests" subdirectory
+                            containing the test scenarios in Bash
+                id -- full rule id as it is present in datastream
+                short_id -- short rule ID, the same as basename of the directory
+                            containing the test scenarios in Bash
+                template -- name of the template the rule uses
+                local_env_yaml -- env_yaml specific to rule's own context
+                rule -- rule class, contains information parsed from rule.yml
+        """
 
-        return rules_to_test
+        # Here we need to perform some magic to handle parsing the rule (from a
+        # product perspective) and loading any templated tests. In particular,
+        # identifying which tests to potentially run involves invoking the
+        # templating engine.
+        #
+        # Begin by loading context about our execution environment, if any.
+        product = self.test_env.product
+        product_yaml = common.get_product_context(product)
+        all_rules_in_benchmark = xml_operations.get_all_rules_in_benchmark(
+            self.datastream, self.benchmark_id)
+        rules = []
+
+        for dirpath, dirnames, filenames in common.walk_through_benchmark_dirs(
+                product):
+            if not common.is_rule_dir(dirpath):
+                continue
+            short_rule_id = os.path.basename(dirpath)
+            full_rule_id = OSCAP_RULE + short_rule_id
+            if not self._rule_matches_rule_spec(short_rule_id):
+                continue
+            if full_rule_id not in all_rules_in_benchmark:
+                # This is an error only if the user specified the rules to be
+                # tested explicitly using command line arguments
+                if self.target_type == "rule ID":
+                    logging.error(
+                        "Rule '{0}' isn't present in benchmark '{1}' in '{2}'"
+                        .format(
+                            full_rule_id, self.benchmark_id, self.datastream))
+                continue
+
+            # Load the rule itself to check for a template.
+            rule, local_env_yaml = common.load_rule_and_env(
+                dirpath, product_yaml, product)
+
+            # Before we get too far, we wish to search the rule YAML to see if
+            # it is applicable to the current product. If we have a product
+            # and the rule isn't applicable for the product, there's no point
+            # in continuing with the rest of the loading. This should speed up
+            # the loading of the templated tests. Note that we've already
+            # parsed the prodtype into local_env_yaml
+            if product and local_env_yaml['products']:
+                prodtypes = local_env_yaml['products']
+                if "all" not in prodtypes and product not in prodtypes:
+                    continue
+
+            tests_dir = os.path.join(dirpath, "tests")
+            template_name = None
+            if rule.template and rule.template['vars']:
+                template_name = rule.template['name']
+            if not self._rule_matches_template_spec(template_name):
+                continue
+            result = Rule(
+                directory=tests_dir, id=full_rule_id,
+                short_id=short_rule_id, template=template_name,
+                local_env_yaml=local_env_yaml, rule=rule)
+            rules.append(result)
+        return rules
 
     def test_rule(self, state, rule, scenarios):
         remediation_available = self._is_remediation_available(rule)
@@ -288,13 +336,14 @@ class RuleChecker(oscap.Checker):
             rule, scenarios,
             self.remote_dir, state, remediation_available)
 
-    def _slice_sbr(self, scenarios_by_rule_id, slice_current, slice_total):
+    def _slice_sbr(self, test_content_by_rule_id, slice_current, slice_total):
         """  Returns only a subset of test scenarios, representing slice_current-th
         slice out of slice_total"""
 
         tuple_repr = []
-        for rule_id in scenarios_by_rule_id:
-            tuple_repr += itertools.product([rule_id], scenarios_by_rule_id[rule_id])
+        for rule_id in test_content_by_rule_id:
+            tuple_repr += itertools.product(
+                [rule_id], test_content_by_rule_id[rule_id].scenarios)
 
         total_scenarios = len(tuple_repr)
         slice_low_bound = math.ceil(total_scenarios / slice_total * (slice_current - 1))
@@ -303,96 +352,102 @@ class RuleChecker(oscap.Checker):
         new_sbr = {}
         for rule_id, scenario in tuple_repr[slice_low_bound:slice_high_bound]:
             try:
-                new_sbr[rule_id].append(scenario)
+                new_sbr[rule_id].scenarios.append(scenario)
             except KeyError:
-                new_sbr[rule_id] = [scenario]
+                scenarios = [scenario]
+                other_content = test_content_by_rule_id[rule_id].other_content
+                new_sbr[rule_id] = RuleTestContent(scenarios, other_content)
         return new_sbr
 
-    def _test_target(self, target):
-        rules_to_test = self._get_rules_to_test(target)
+    def _find_tests_paths(self, rule, product_yaml):
+        # Start by checking for templating tests
+        templated_tests_paths = common.fetch_templated_tests_paths(
+            rule, product_yaml)
+
+        # Add additional tests from the local rule directory. Note that,
+        # like the behavior in template_tests, this will overwrite any
+        # templated tests with the same file name.
+        local_tests_paths = common.fetch_local_tests_paths(rule.directory)
+
+        for filename in local_tests_paths:
+            templated_tests_paths.pop(filename, None)
+        if self.target_type != "template" and not self.test_env.duplicate_templates:
+            for filename in self.used_templated_test_scenarios[rule.template]:
+                templated_tests_paths.pop(filename, None)
+            self.used_templated_test_scenarios[rule.template] |= set(
+                templated_tests_paths.keys())
+        return templated_tests_paths.values(), local_tests_paths.values()
+
+    def _load_all_tests(self, rule):
+        product_yaml = common.get_product_context(self.test_env.product)
+
+        templated_tests_paths, local_tests_paths = self._find_tests_paths(
+            rule, product_yaml)
+
+        # All tests is a mapping from path (in the tarball) to contents
+        # of the test case. This is necessary because later code (which
+        # attempts to parse headers from the test case) don't have easy
+        # access to templated content. By reading it and returning it
+        # here, we can save later code from having to understand the
+        # templating system.
+        all_tests = dict()
+        templated_tests = common.load_templated_tests(
+            templated_tests_paths, rule.rule.template,
+            rule.local_env_yaml)
+        local_tests = common.load_local_tests(
+            local_tests_paths, rule.local_env_yaml)
+        all_tests.update(templated_tests)
+        all_tests.update(local_tests)
+        return all_tests
+
+    def _get_rule_test_content(self, rule):
+        all_tests = self._load_all_tests(rule)
+        scenarios = []
+        other_content = dict()
+        for file_name, file_content in all_tests.items():
+            scenario_matches_regex = r'.*\.[^.]*\.sh$'
+            if re.search(scenario_matches_regex, file_name):
+                scenario = Scenario(file_name, file_content)
+                scenario.override_profile(self.scenarios_profile)
+                if scenario.matches_regex_and_platform(
+                        self.scenarios_regex, self.benchmark_cpes):
+                    scenarios.append(scenario)
+            else:
+                other_content[file_name] = file_content
+        return RuleTestContent(scenarios, other_content)
+
+    def _get_test_content_by_rule_id(self, rules_to_test):
+        test_content_by_rule_id = dict()
+        for rule in rules_to_test:
+            rule_test_content = self._get_rule_test_content(rule)
+            test_content_by_rule_id[rule.id] = rule_test_content
+        sliced_test_content_by_rule_id = self._slice_sbr(
+            test_content_by_rule_id, self.slice_current, self.slice_total)
+        return sliced_test_content_by_rule_id
+
+    def _test_target(self):
+        rules_to_test = self._get_rules_to_test()
         if not rules_to_test:
             logging.error("No tests found matching the {0}(s) '{1}'".format(
                 self.target_type,
-                ", ".join(target)))
+                ", ".join(self.rule_spec)))
             return
 
-        scenarios_by_rule_id = dict()
-        for rule in rules_to_test:
-            rule_scenarios = self._get_scenarios(
-                rule.directory, rule.scenarios_basenames, self.scenarios_regex,
-                self.benchmark_cpes)
-            scenarios_by_rule_id[rule.id] = rule_scenarios
-        sliced_scenarios_by_rule_id = self._slice_sbr(scenarios_by_rule_id,
-                                                      self.slice_current,
-                                                      self.slice_total)
-        self._prepare_environment(sliced_scenarios_by_rule_id)
+        test_content_by_rule_id = self._get_test_content_by_rule_id(
+            rules_to_test)
 
-        with test_env.SavedState.create_from_environment(self.test_env, "tests_uploaded") as state:
+        self._prepare_environment(test_content_by_rule_id)
+
+        with test_env.SavedState.create_from_environment(
+                self.test_env, "tests_uploaded") as state:
             for rule in rules_to_test:
                 try:
-                    self.test_rule(state, rule, sliced_scenarios_by_rule_id[rule.id])
+                    scenarios = test_content_by_rule_id[rule.id].scenarios
+                    self.test_rule(state, rule, scenarios)
                 except KeyError:
                     # rule is not processed in given slice
                     pass
 
-    def _modify_parameters(self, script, params):
-        if self.scenarios_profile:
-            params['profiles'] = [self.scenarios_profile]
-
-        if not params["profiles"]:
-            params["profiles"].append(OSCAP_PROFILE_ALL_ID)
-            logging.debug(
-                "Added the {0} profile to the list of available profiles for {1}"
-                .format(OSCAP_PROFILE_ALL_ID, script))
-        return params
-
-    def _parse_parameters(self, script_content):
-        """Parse parameters from script header"""
-        params = {'profiles': [],
-                  'templates': [],
-                  'packages': [],
-                  'platform': ['multi_platform_all'],
-                  'remediation': ['all'],
-                  'variables': [],
-                  }
-
-        for parameter in params:
-            found = re.search(r'^# {0} = (.*)$'.format(parameter),
-                              script_content, re.MULTILINE)
-            if found is None:
-                continue
-            splitted = found.group(1).split(',')
-            params[parameter] = [value.strip() for value in splitted]
-
-        return params
-
-    def _get_scenarios(self, rule_dir, scripts, scenarios_regex, benchmark_cpes):
-        """ Returns only valid scenario files, rest is ignored (is not meant
-        to be executed directly.
-        """
-
-        if scenarios_regex is not None:
-            scenarios_pattern = re.compile(scenarios_regex)
-
-        scenarios = []
-        for script in scripts:
-            script_contents = scripts[script]
-            if scenarios_regex is not None:
-                if scenarios_pattern.match(script) is None:
-                    logging.debug("Skipping script %s - it did not match "
-                                  "--scenarios regex" % script)
-                    continue
-            script_context = _get_script_context(script)
-            if script_context is not None:
-                script_params = self._parse_parameters(script_contents)
-                script_params = self._modify_parameters(script, script_params)
-                if common.matches_platform(script_params["platform"], benchmark_cpes):
-                    scenarios += [Scenario(script, script_context, script_params, script_contents)]
-                else:
-                    logging.warning("Script %s is not applicable on given platform" % script)
-
-        return scenarios
-
     def _check_rule(self, rule, scenarios, remote_dir, state, remediation_available):
         remote_rule_dir = os.path.join(remote_dir, rule.short_id)
         logging.info(rule.id)
@@ -431,7 +486,8 @@ class RuleChecker(oscap.Checker):
         os.unlink(new_filename)
 
     def _change_variable_value(self, varname, value):
-        _, xslt_filename = tempfile.mkstemp(prefix="xslt-change-value", dir="/tmp")
+        descriptor, xslt_filename = tempfile.mkstemp(prefix="xslt-change-value", dir="/tmp")
+        os.close(descriptor)
         template = generate_xslt_change_value_template(varname, value)
         with open(xslt_filename, "w") as fp:
             fp.write(template)
@@ -444,7 +500,7 @@ class RuleChecker(oscap.Checker):
                     log_file)
             if result.returncode:
                 msg = (
-                    "Error changing value of '{varname}': {stdout}"
+                    "Error changing value of '{varname}': {stderr}"
                     .format(varname=varname, stderr=result.stderr)
                 )
                 raise RuntimeError(msg)
@@ -489,6 +545,92 @@ class RuleChecker(oscap.Checker):
             json.dump(self.results, f)
 
 
+class Scenario():
+    def __init__(self, script, script_contents):
+        self.script = script
+        self.context = self._get_script_context()
+        self.contents = script_contents
+        self.script_params = self._parse_parameters()
+
+    def _get_script_context(self):
+        """Return context of the script."""
+        result = re.search(r'.*\.([^.]*)\.[^.]*$', self.script)
+        if result is None:
+            return None
+        return result.group(1)
+
+    def _parse_parameters(self):
+        """Parse parameters from script header"""
+        params = {
+            'profiles': [],
+            'templates': [],
+            'packages': [],
+            'platform': ['multi_platform_all'],
+            'remediation': ['all'],
+            'variables': [],
+        }
+
+        for parameter in params:
+            found = re.search(
+                r'^# {0} = (.*)$'.format(parameter),
+                self.contents, re.MULTILINE)
+            if found is None:
+                continue
+            if parameter == "variables":
+                variables = []
+                for token in found.group(1).split(','):
+                    token = token.strip()
+                    if '=' in token:
+                        variables.append(token)
+                    else:
+                        variables[-1] += "," + token
+                params["variables"] = variables
+            else:
+                splitted = found.group(1).split(',')
+                params[parameter] = [value.strip() for value in splitted]
+
+        if not params["profiles"]:
+            params["profiles"].append(OSCAP_PROFILE_ALL_ID)
+            logging.debug(
+                "Added the {0} profile to the list of available profiles "
+                "for {1}"
+                .format(OSCAP_PROFILE_ALL_ID, self.script))
+
+        return params
+
+    def override_profile(self, scenarios_profile):
+        if scenarios_profile:
+            self.script_params['profiles'] = [scenarios_profile]
+
+    def matches_regex(self, scenarios_regex):
+        if scenarios_regex is not None:
+            scenarios_pattern = re.compile(scenarios_regex)
+            if scenarios_pattern.match(self.script) is None:
+                logging.debug(
+                    "Skipping script %s - it did not match "
+                    "--scenarios regex" % self.script
+                )
+                return False
+        return True
+
+    def matches_platform(self, benchmark_cpes):
+        if self.context is None:
+            return False
+        if common.matches_platform(
+                self.script_params["platform"], benchmark_cpes):
+            return True
+        else:
+            logging.warning(
+                "Script %s is not applicable on given platform" %
+                self.script)
+            return False
+
+    def matches_regex_and_platform(self, scenarios_regex, benchmark_cpes):
+        return (
+            self.matches_regex(scenarios_regex)
+            and self.matches_platform(benchmark_cpes))
+
+
 def perform_rule_check(options):
     checker = RuleChecker(options.test_env)
 
@@ -503,12 +645,12 @@ def perform_rule_check(options):
     checker.slice_current = options.slice_current
     checker.slice_total = options.slice_total
     checker.keep_snapshots = options.keep_snapshots
-
+    checker.rule_spec = options.target
+    checker.template_spec = None
     checker.scenarios_profile = options.scenarios_profile
     # check if target is a complete profile ID, if not prepend profile prefix
     if (checker.scenarios_profile is not None and
             not checker.scenarios_profile.startswith(OSCAP_PROFILE) and
             not oscap.is_virtual_oscap_profile(checker.scenarios_profile)):
         checker.scenarios_profile = OSCAP_PROFILE+options.scenarios_profile
-
-    checker.test_target(options.target)
+    checker.test_target()
